from Python import Ss7Python as SS7
from typing import Any
from docx.enum.text import WD_ALIGN_PARAGRAPH
from docx.shared import Inches
from docx.shared import RGBColor

import csv
import os
import docx


##################################################################
class Brace_Data:
    def __init__(
        self: Any,
        Fugo: str,
        Floor: str,
        Frame: str,
        Axis1: str,
        Axis2: str,
        case: str,
        LR: str,
        Rate: float,
    ) -> None:
        # 符号ごとの代表ブレースデータ
        self.Fugo = Fugo
        self.Floor = Floor
        self.Frame = Frame
        self.Axis1 = Axis1
        self.Axis2 = Axis2
        self.case = case
        self.LR = LR
        self.Rate = Rate


##################################################################
def import_csv(file_path: str) -> list:
    """CSVの読み込み

    CSVの情報を取得します。
    Args:
        file_path (str): CSVファイルのパス
    Returns:
        list: CSVファイルの中身のリスト

    """

    data = []
    # CSVファイルを読み込む
    with open(file_path, mode="r", encoding="cp932") as file:
        reader = csv.reader(file)
        for row in reader:
            data.append(row)

    return data


##################################################################
def search_csv(data: list, target: str) -> list:
    """文字列検索

    listの中から指定した文字列の行番号を取得できます。
    Args:
        data (list): 文字列のリスト
        target (str): 検索したい文字列
    Returns:
        int: 行番号

    """

    line_no = []  # 行番号
    line_no = [i for i, line_s in enumerate(data) if target in line_s]
    return line_no


##################################################################
def export_res_csv(Data: Any, result: str, folder_path: str, target: str) -> None:
    """結果CSVのエクスポート
    Args:
        Data: SS7のデータ
        result (str): 結果セット名
        folder_path (str): 結果CSVを出力するフォルダのパス
        target (str): 出力する出力項目名
    """
    # 計算結果データを取得する
    res = Data.GetResultData(result)

    # 結果CSVをエクスポートする
    result_path = os.path.abspath(folder_path + "\\result.csv")

    # 2=中断する 1=省略する 1=全部材出力
    res.ExportResultCsv(target, result_path, 1, 1, 1)

    # 結果CSVのエクスポートが正しく実行できたことの判定
    err = SS7.GetLastError()
    if err.IsOK() is False:
        print("結果CSVのエクスポートが失敗しました")


#################################################################
def set_brace_section(file_path: str) -> list:
    """ブレース断面形状のセット
    Args:
        file_path (str): CSVファイルのパス
    Returns:
        BSection (str): 符号毎のブレース断面形状
    """
    data = import_csv(file_path)
    line_no = search_csv(data, "name=鉛直ブレース断面リスト")
    offset = line_no[0]

    BSection = []
    DataExist = True

    i = 0
    # データがなくなるまでループ
    while DataExist:

        # 部材がなくなったかの判定
        iLen = len(data[offset + 4 + i])
        if iLen < 1:
            break

        Fugo = data[offset + 4 + i][0]
        Section = data[offset + 4 + i][1]
        Mate = data[offset + 4 + i][2]

        BSection.append([Fugo, Section, Mate])
        i += 1

    return BSection


#################################################################
def set_brace_data(file_path: str) -> list:
    """ブレースデータのセット
    Args:
        file_path (str): CSVファイルのパス
    Returns:
        BdataList (Brace_Data): 階・符号毎のブレースデータまとめ
    """

    data = import_csv(file_path)
    line_no = search_csv(data, "name=鉛直ブレース検定比一覧(ケース・部材ごと)")
    BdataList: list = []

    # データの数
    data_size = len(data)

    DataExist = True
    i = 0
    Fugo_count = 0

    # ケースごとのループ
    for case_no in line_no:
        # ケース名
        case_name = data[case_no][1].replace("case=", "")
        # 部材がなくなるまでループ
        while DataExist:
            # 終了判定
            if data_size - 1 < (case_no + 4 + i):
                break
            # 部材がなくなったかの判定
            iLen = len(data[case_no + 4 + i])
            if iLen < 1:
                # ケース終了（次のケースへ）
                i = 0
                break

            # 位置
            Floor = data[case_no + 4 + i][0]
            Frame = data[case_no + 4 + i][1]
            Axis1 = data[case_no + 4 + i][2]
            Axis2 = data[case_no + 4 + i][3]
            # 符号
            Fugo = data[case_no + 4 + i][4]
            # 検定比
            RateNCL = data[case_no + 4 + i][5]
            RateNCR = data[case_no + 4 + i][6]
            RateNTL = data[case_no + 4 + i][7]
            RateNTR = data[case_no + 4 + i][8]
            # 最大検定比
            maxRate = max(RateNCL, RateNCR, RateNTL, RateNTR)
            # 左右の判別
            if max(RateNCL, RateNTL) > max(RateNCR, RateNTR):
                LR = "左下り"
            else:
                LR = "右下り"

            # リストに登録済みの符号かどうか
            Fugo_Exist = False

            # リスト内に同じ符号名があるか検索
            if Fugo_count > 0:
                for item in BdataList:
                    # 同じ符号、同じ階が見つかった
                    if Fugo == item.Fugo and Floor == item.Floor:
                        # 符号存在フラグ
                        Fugo_Exist = True
                        # 最大検定比の比較
                        if maxRate > item.Rate:
                            item.case = case_name
                            item.LR = LR
                            item.Floor = Floor
                            item.Frame = Frame
                            item.Axis1 = Axis1
                            item.Axis2 = Axis2
                            item.Rate = maxRate
                        break

            # 同じ符号が見つからなかった
            if not Fugo_Exist:
                BdataList.append(Brace_Data("", "", "", "", "", "", "", 0.0))
                BdataList[Fugo_count].case = case_name
                BdataList[Fugo_count].LR = LR
                BdataList[Fugo_count].Fugo = Fugo
                BdataList[Fugo_count].Floor = Floor
                BdataList[Fugo_count].Frame = Frame
                BdataList[Fugo_count].Axis1 = Axis1
                BdataList[Fugo_count].Axis2 = Axis2
                BdataList[Fugo_count].Rate = maxRate
                Fugo_count += 1

            # 次の行へ移動
            i += 1

        # 部材ごとのループ
    # ケースごとのループ

    return BdataList


#################################################################
def output_docx(BSection: list, BdataList: list, data_path: str) -> None:
    """docxへの出力
    Args:
        BSection (list): 符号毎のブレース断面形状
        BdataList (list): 符号毎のブレースデータ
        data_path (str): 物件データのパス
    """

    # ドキュメントを新規作成
    doc = docx.Document()
    # 出力ファイル
    out_path = data_path + "\\Brace_Rate.docx"
    # タイトル
    doc.add_paragraph("ブレース検定比一覧", "Heading 1")

    # 表
    tbl = doc.add_table(rows=1, cols=7, style="Table Grid")
    row = tbl.rows[0]
    row.cells[0].text = "階"
    row.cells[1].text = "符号"
    row.cells[2].text = "配置"
    row.cells[3].text = "断面"
    row.cells[4].text = "ケース"
    row.cells[5].text = "左右"
    row.cells[6].text = "検定比"

    i = 0
    for Brace in BdataList:
        # フレーム、軸、軸
        position = Brace.Frame + ", " + Brace.Axis1 + "-" + Brace.Axis2

        for Section in BSection:
            if Section[0] == Brace.Fugo:
                Section_Name = Section[1]
                break

        cells = tbl.add_row().cells
        cells[0].text = Brace.Floor
        cells[1].text = Brace.Fugo
        cells[2].text = position
        cells[3].text = Section_Name
        cells[4].text = Brace.case
        cells[5].text = Brace.LR
        cells[6].text = Brace.Rate
        i += 1

    # 表のスタイル
    for row in tbl.rows:
        for cell in row.cells:
            paragraphs = cell.paragraphs
            for paragraph in paragraphs:
                paragraph.alignment = WD_ALIGN_PARAGRAPH.CENTER
                for run in paragraph.runs:
                    run.font.size = docx.shared.Pt(9)
                    run.font.name = "ＭＳ Ｐ明朝"

    # 列の幅
    for column in tbl.columns:
        for cell in column.cells:
            cell.width = Inches(0.7)

    for cell in tbl.columns[0].cells:
        cell.width = Inches(0.6)
    for cell in tbl.columns[1].cells:
        cell.width = Inches(0.6)
    for cell in tbl.columns[2].cells:
        cell.width = Inches(1.0)
    for cell in tbl.columns[3].cells:
        cell.width = Inches(1.7)

    # 検定比1.0以上で赤字にする
    for cell in tbl.columns[6].cells:
        paragraphs = cell.paragraphs
        for paragraph in paragraphs:
            # 数値変換できるとき
            if paragraph.text.isascii() is True:
                if float(paragraph.text) >= 1.00:
                    for run in paragraph.runs:
                        run.font.color.rgb = RGBColor(255, 0, 0)  # 赤

    # 保存
    doc.save(out_path)

    return


##################################################################
def main(version: str, data_path: str) -> None:
    """メイン処理
        SS7データを開いて、CSV結果を出力し、ブレースの検定比をdocxに出力する
    Args:
        version (str): SS7のバージョン
        data_path (str): 物件データのパス

    """
    # SS7の起動
    SS7.Init()
    SS7.Start(version, 1)  # 1=ログファイルをクリアする
    # データを開く
    Data = SS7.Open(data_path, 1, 3)  # 1=変換して開く 3=物件データは開かない

    # データが正しく開けたかの判定
    if Data is not None:
        # CSV結果を出力
        export_res_csv(
            Data, "結果1", data_path, "2.6.18.鉛直ブレース;7.2.12.鉛直ブレース"
        )

        # CSV結果の読み込み
        out_path = data_path + "\\result.csv"

        # ブレースリストの作成
        BSection = set_brace_section(out_path)
        BdataList = set_brace_data(out_path)

        # docx出力
        output_docx(BSection, BdataList, data_path)

        return
    else:
        SS7.End(1)  # 1＝保存して終了
        print("物件を開けませんでした")
        return


####################################################
# 変数定義
####################################################
if __name__ == "__main__":

    version = "1.1.1.20"  # SS7のバージョン
    data_path = r"C:\UsrData\SS7Data\sample.ikn"  # 物件フォルダパス

    main(version, data_path)
